import { Component, EventEmitter, Input, OnChanges, OnInit, Output, SimpleChanges } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { forkJoin, lastValueFrom } from 'rxjs';
import { Category } from '../../structure-list/models/category.model';
import { Module } from '../../structure-list/models/module.model';
import { SearchService } from '../../structure-list/services/search.service';
import { SearchQuery } from '../models/searchQuery.model';

@Component({
  selector: 'app-annuaire-header',
  templateUrl: './annuaire-header.component.html',
  styleUrls: ['./annuaire-header.component.scss'],
})
export class AnnuaireHeaderComponent implements OnInit, OnChanges {
  /** Input to reset filters, needed because a reset button is displayed on the result list when it is empty */
  @Input() shouldResetFilters = 0;
  @Output() searchEvent = new EventEmitter<SearchQuery>();

  public jobGroupCategories: Category[] = [];
  public employerCategories: Category[] = [];

  public searchInput = '';
  // Checked modules for each filter (used to receive changes from collapsable-filter)
  public jobGroupCheckedModules: Module[] = [];
  public employerCheckedModules: Module[] = [];

  private jobGroups: string[] = [];
  private employers: string[] = [];
  private jobGroupFilterChecked: string[] = [];
  private employerFilterChecked: string[] = [];

  constructor(
    private activatedRoute: ActivatedRoute,
    private router: Router,
    public searchService: SearchService,
  ) {}

  async ngOnInit(): Promise<void> {
    this.loadQueryParams();
    // Will store the different categories
    await this.loadFilters();

    if (this.searchService.previousResult$.getValue().docs.length === 0) {
      this.applyFilter();
    }
  }

  ngOnChanges(changes: SimpleChanges): void {
    if (changes.shouldResetFilters?.currentValue !== 0) {
      this.resetFilters();
    }
  }

  private loadQueryParams(): void {
    const queryParams = this.activatedRoute.snapshot.queryParams;
    this.searchInput = queryParams['search'] ?? '';
    this.jobGroupFilterChecked = queryParams['jobs']?.split('|') ?? [];
    this.employerFilterChecked = queryParams['employers']?.split('|') ?? [];
    this.searchService.checkedFilterList = [...this.jobGroupFilterChecked, ...this.employerFilterChecked];
    this.searchService.annuaireSearchQuery = {
      search: this.searchInput,
      page: Number(queryParams['page']) || 1,
      jobFilters: this.jobGroupFilterChecked,
      employerFilters: this.employerFilterChecked,
    };
  }

  public onSearchSubmitted(value: string): void {
    this.searchInput = value;
    this.applyFilter();
  }

  private updateQueryParams(): void {
    const queryParams: Record<string, string> = {};

    if (this.searchInput.trim()) {
      queryParams.search = this.searchInput.trim();
    }

    if (this.jobGroupFilterChecked.length) {
      queryParams.jobs = this.jobGroupFilterChecked.join('|');
    }

    if (this.employerFilterChecked.length) {
      queryParams.employers = this.employerFilterChecked.join('|');
    }

    // Delay to avoid error in console: "NG0100: ExpressionChangedAfterItHasBeenCheckedError"
    setTimeout(
      () =>
        this.router.navigate([], {
          relativeTo: this.activatedRoute,
          queryParams: queryParams,
        }),
      1,
    );
  }

  public applyFilter(page = 1): void {
    this.jobGroupCheckedModules = this.jobGroupFilterChecked.map((filter) => {
      return new Module(filter, 'jobGroup', filter);
    });
    this.employerCheckedModules = this.employerFilterChecked.map((filter) => {
      return new Module(filter, 'employer', filter);
    });

    this.searchEvent.emit({
      search: this.searchInput.trim(),
      page: page,
      jobFilters: this.jobGroupFilterChecked,
      employerFilters: this.employerFilterChecked,
    });

    this.updateQueryParams();
  }

  /** Get the categories for each modal type */
  private async loadFilters(): Promise<void> {
    const { jobGroups, employers } = await lastValueFrom(
      forkJoin({
        jobGroups: this.searchService.getJobsGroups(),
        employers: this.searchService.getEmployers(),
      }),
    );
    this.jobGroups = jobGroups;
    this.employers = employers;
    this.jobGroupCategories = [
      {
        id: 'jobGroup',
        name: 'Fonction',
        theme: 'Fonction',
        modules: [],
      },
    ];
    jobGroups.forEach((jobGroup) => {
      this.jobGroupCategories[0].modules.push({
        disabled: false,
        name: jobGroup,
        id: jobGroup,
      });
    });

    this.employerCategories = [
      {
        id: 'employer',
        name: 'Employeur',
        theme: 'Employeur',
        modules: [],
      },
    ];
    employers.forEach((employer) => {
      this.employerCategories[0].modules.push({
        disabled: false,
        name: employer,
        id: employer,
      });
    });
  }

  private splitFilters(checkedFilterList: string[]): void {
    this.jobGroupFilterChecked = checkedFilterList.filter((filter) => this.jobGroups.includes(filter));
    this.employerFilterChecked = checkedFilterList.filter((filter) => this.employers.includes(filter));
  }

  public onSelectJobGroups(jobGroupCheckedModules: Module[]): void {
    this.jobGroupFilterChecked = jobGroupCheckedModules.map((module) => module.id);
    this.searchService.checkedFilterList = [...this.jobGroupFilterChecked, ...this.employerFilterChecked];
    this.applyFilter();
  }

  public onSelectEmployers(employerCheckedModules: Module[]): void {
    this.employerFilterChecked = employerCheckedModules.map((module) => module.id);
    this.searchService.checkedFilterList = [...this.jobGroupFilterChecked, ...this.employerFilterChecked];
    this.applyFilter();
  }

  public resetFilters(): void {
    this.searchInput = '';
    this.searchService.checkedFilterList = [];
    this.jobGroupFilterChecked = [];
    this.employerFilterChecked = [];
    this.applyFilter();
  }

  public removeFilter(filter: string): void {
    const index = this.searchService.checkedFilterList.findIndex((checkedFilter: string) => checkedFilter === filter);
    if (index !== -1) {
      this.searchService.checkedFilterList.splice(index, 1);
      this.splitFilters(this.searchService.checkedFilterList);
      this.updateQueryParams();
      this.applyFilter();
    }
  }
}
